// -*-C++-*-
//
// File:      DalListView.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Tue Jul 25 14:56:37 MET DST 2000

#include "DalListView.h"
#include "DalListViewItem.h"
#include "DalListViewFolder.h"

#include <qpopupmenu.h>
#include <qtooltip.h>



DalListView::DalListView(bool tree,QWidget* parent, const char* name) :
  QListView(parent,name),
  _tree(tree),
  _type(Any),
  _root(0)
{
  // setup signal forwarding:
  connect(this,SIGNAL(selectionChanged(QListViewItem*)),
	  this,SLOT(handleSelectionChanged(QListViewItem*)));

  connect(this,SIGNAL(rightButtonPressed(QListViewItem*,const QPoint&,int)),
	  this,SLOT(handleRightClicked(QListViewItem*,const QPoint&,int)));

  connect(this,SIGNAL(doubleClicked(QListViewItem*)),
	  this,SLOT(handleDoubleClicked(QListViewItem*)));
  connect(this,SIGNAL(returnPressed(QListViewItem*)),
	  this,SLOT(handleDoubleClicked(QListViewItem*)));

  if (_tree) {
    _root=new Folder("/");
    addColumn("");
    setColumnWidth(0,200);
    //setRootIsDecorated(false);
    setRootIsDecorated(true);
 
    DalListViewItem* current=0;
    current=new DalListViewFolder(_root,this);
  }
  else {
    QToolTip::add(this,
		  "Left-click on item to select\n"
		  "Double-click on item to open\n"
		  "Right-click on item to get menu");
    addColumn("Name");
    addColumn("Type");
    setSorting(1,false);
    //    setMultiSelection(true);
  }

}

DalListView::~DalListView()
{
  clear();
  if (_root) delete _root;
}

QString DalListView::selectedQualifiedName()
{
  DalListViewItem* d=selectedItem();
  if (d) return d->qualifiedName();
  else return QString();
}

DalListViewItem* DalListView::openQualifiedName(const QString& n)
{
  /* browse all siblings to find the best matching root item
   */

  DalListViewItem* d=firstChild();
  DalListViewItem* best=0;
  int idxBest=-1;

  while (d) {
    QString cur=d->qualifiedName();
    if (cur == n) return d;

    int idxCur=n.find(cur);
    if ( idxCur != -1) { // it is a match: but better than last?
      if (idxCur > idxBest) {
	idxBest=idxCur;
	best=d;
      }
    }

    d=d->nextSibling();
  }

  if (best) return best->open(n);
  return (DalListViewItem*)0;
}


/*
  Tree only!
 */
void DalListView::handleLeftClicked (DalListViewItem * d)
{

  if (!d) return;
  if (!_tree) return;
  
  DalListViewItem* current=currentItem();
  if (!current) return; //something very wrong?
  current->setOpen(true);
  
  current=current->firstChild();
  while ( current !=0 ) {
    if (current->text(0) == d->text(0)) {
      current->setOpen(true);
      setCurrentItem(current);
      setSelected(current,true);
      ensureItemVisible(current);
      break;
    }
    current = current->nextSibling();
  }
}

/*
  Flat only! Open up the selection and show its contents
 */
void DalListView::handleDoubleClicked(DalListViewItem* i)
{
  if (!i) return;
  if (_tree) return;
  
  i->openChildren(this);
}

void DalListView::gotoParent()
{
  if (!_tree) return;
  
  DalListViewItem* item=selectedItem();
  if (!item) return;

  item = item->parent();
  if (!item) return;

  ensureItemVisible(item);
  setSelected(item,true);
}

void DalListView::jumpTo(DalListViewItem* item)
{
  if (!item)  return;
  if (!_tree) return;
  ensureItemVisible(item);
  setSelected(item,true);  
}


/* ----------------------------------------------------------------------------
  converters/forwarders
 */

DalListViewItem* DalListView::currentItem()
{
  return convert(QListView::currentItem());
}

DalListViewItem* DalListView::selectedItem()
{
  return convert(QListView::selectedItem());
}

DalListViewItem* DalListView::firstChild()
{
  return convert(QListView::firstChild());
}


/* ----------------------------------------------------------------------------
  Private slots: only used for forwarding real items
    Unfortunately, we !have! to cast the items ;-(

    But be safe, do dynamic_cast only!!!
 */

void DalListView::handleSelectionChanged(QListViewItem* i)
{
  if (DalListViewItem* d=convert(i)) {
    // flat view: can select?
    if (!_tree) {
      if (_type == Any || d->type() == _type) emit addSelection(d);
    }
    emit leftClicked(d);
  }
}

void DalListView::handleDoubleClicked(QListViewItem* i)
{
  if (DalListViewItem* d=convert(i))
    emit doubleClicked(d);
}

void DalListView::handleRightClicked(QListViewItem* i,const QPoint& atPos,int)
{

  DalListViewItem* d=convert(i);
  if (!d) return;
  setSelected(d,true);

  if (_tree) return; // popup only allowed in flat pane


  QPopupMenu menu(this);
  menu.insertItem("&Open",1);
  menu.insertItem("&Select",2);

  /* NOT IMPLEMENTED
     menu.insertSeparator();
     menu.insertItem("&Cut",3);
     menu.insertItem("&Copy",4);
     menu.insertItem("&Paste",5);
     menu.insertSeparator();
     menu.insertItem("&Rename",6);
  */
  
  // enable/disable items in menu
  if (_type != Any) {
    if (d->type() != _type)  menu.setItemEnabled(2,false);
  }

  /* NOT IMPLEMENTED
     menu.setItemEnabled(3,false);
     menu.setItemEnabled(4,false);
     menu.setItemEnabled(5,false);
     menu.setItemEnabled(6,false);
  */

  //
  menu.setMouseTracking(true);  
  switch (menu.exec(atPos)) {
  case 1:
    emit doubleClicked(d);
    break;
  case 2:
    emit addSelection(d);
    break;
  }    
}

DalListViewItem* DalListView::convert(QListViewItem* i)
{
  return dynamic_cast<DalListViewItem*>(i);
}
