/*****************************************************************************
 *
 *  GuiBuilder.cc                               XMM Science Analysis Subsystem
 *
 *  Plug-in classes for GUI layout
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: GuiBuilder.cc,v 1.14 2000/12/12 11:08:54 jbakker Exp $
 *
 *****************************************************************************/

#include "GuiBuilder.h"
#include "Parameter.h"
#include "Layout.h"
#include "ParamWidget.h"
#include "BoolParamWidget.h"
#include "StringParamWidget.h"
#include "LabelFrame.h"
#include "EnableFrame.h"
#include "ChoiceFrame.h"
#include "EnablingWidget.h"
#include "TabCardWidget.h"

#include <qwidget.h>
#include <qlabel.h>
#include <qgroupbox.h>
#include <qtabwidget.h>
#include <qlayout.h>

#include <iostream.h>


//================================================================
//  Class GuiBuilder
//================================================================

// Constants
const int GuiBuilder::_margin =  5;
const int GuiBuilder::_spacing = 0;

/*** Construct a single parameter widget with a label ***/

void GuiBuilder::buildSimple(Layout* layout)
{
    SimpleLayout* s = (SimpleLayout*)layout;
    Parameter* p = s->param();
    const char* name = p->name().c_str();

    EnablingWidget* canvas = new EnablingWidget(_parent, name);

    QLabel* label = new QLabel(name,canvas);
    label->setFixedSize(_labelSize);

    QWidget* w = ParamWidget::makeWidget(p,canvas);

    QHBoxLayout* box = new QHBoxLayout(canvas,0,_spacing);
    box->addSpacing(_margin);
    box->addWidget(label);
    box->addSpacing(_margin);
    if(w)
    {

      bool stretch=false;
      switch(p->getType()){
      case Parameter::Bool : {
	//case Parameter::Time :
	//case Parameter::Angle :
	//case Parameter::StringChoice:
	break;
      }
      case Parameter::StringChoice: {
	QLineEdit le("Txt",0);
	w->setFixedSize(w->sizeHint().width(),le.minimumSizeHint().height());
	stretch=true;
	break;
      }
      case Parameter::Int :
      case Parameter::Real :
	w->setFixedWidth(100);
	stretch=true;
	break;	
	
	//      case Parameter:: :
	//       	break;
      default: {
	  QLineEdit le("Txt",0);
	  QSize sz = le.minimumSizeHint();
	  w->setMinimumWidth(150);
	  w->setFixedHeight(sz.height());
	  break;
	}
      }
      box->addWidget(w);
      if (stretch) box->addStretch(1);

    }
    box->addSpacing(_margin);
    _widget = canvas;
}


/*** Construct a frame containing child widgets ***/

void GuiBuilder::buildFrame(Layout* layout)
{    
    FrameLayout* p = (FrameLayout*)layout;
    LabelFrame* frame = new LabelFrame(_parent,"EnableFrame");
    
    frame->setLabel(p->name().c_str());
    list<Layout*>::const_iterator it = p->items().begin();
    while(it != p->items().end())
    {
        GuiBuilder* b = new GuiBuilder(frame->contents(),_labelSize);
        (*it++)->build(b);
	frame->addWidget(b->widget());
    }
    if (frame->layout()) frame->layout()->setMargin(_margin);
    _widget = frame;
}

/*** Construct a set of tab cards ***/

void GuiBuilder::buildCards(Layout* layout) 
{
    PagedLayout* p = (PagedLayout*)layout;
    TabCardWidget* frame = new TabCardWidget(_parent);

    list<Layout*>::const_iterator it = p->items().begin();
    while(it != p->items().end())
    {
      const char* name = (*it)->name().c_str();

      EnablingWidget* card = new EnablingWidget(frame,"cardsEW");
      QVBoxLayout* box = new QVBoxLayout(card,_margin,_spacing);

      GuiBuilder* b = new GuiBuilder(card,_labelSize);
      (*it++)->build(b);
      //box->addSpacing(10);
      box->addWidget(b->widget());
      //box->addStretch(1);
      //box->addSpacing(10);
      frame->addTab(card,name);
      box->addStretch(1);
    }
    if (frame->layout()) frame->layout()->setMargin(_margin);
    _widget = frame;
}


/*** Lay out child widgets in a row ***/

void GuiBuilder::buildRow(Layout* layout) 
{
    RowLayout* p = (RowLayout*)layout;

    EnablingWidget* row = new EnablingWidget(_parent,"rowEW");
    QHBoxLayout* box = new QHBoxLayout(row,0,_spacing);

    list<Layout*>::const_iterator it = p->items().begin();
    while(it != p->items().end())
      {
        GuiBuilder* b = new GuiBuilder(row,_labelSize);
        (*it++)->build(b);
	box->addWidget(b->widget());
    }
    _widget = row;
}
 

/*** Lay out child widgets in a column ***/

void GuiBuilder::buildColumn(Layout* layout) 
{
    ColumnLayout* p = (ColumnLayout*)layout;

    EnablingWidget* col = new EnablingWidget(_parent,"colEW");
    QVBoxLayout* box = new QVBoxLayout(col,0,_spacing);

    list<Layout*>::const_iterator it = p->items().begin();
    while(it != p->items().end())
    {
        GuiBuilder* b = new GuiBuilder(col,_labelSize);
        (*it++)->build(b);
	box->addWidget(b->widget());
    }
    _widget = col;
}


/*** Construct a frame with control to enable child widgets ***/

void GuiBuilder::buildEnable(Layout* layout) 
{
  EnableLayout* p = (EnableLayout*)layout;
  
  Parameter* cntlPar = p->control();
  
  // Note: LYTReader should not allow wrong control type
  if(cntlPar->getType() == Parameter::Bool) {

    EnableFrame* frame = new EnableFrame(_parent);
    
    list<Layout*>::const_iterator it = p->items().begin();
    while(it != p->items().end()) {
      GuiBuilder* b = new GuiBuilder(frame->contents(),_labelSize);
      (*it++)->build(b);
      frame->addWidget(b->widget());
      
    }
    frame->setLabel((BoolParameter*)cntlPar);
    if (frame->layout()) frame->layout()->setMargin(_margin);
    _widget = frame;
  }
}


/*** Build a frame with a control to show sets of child widgets ***/

void GuiBuilder::buildChoice(Layout* layout)
{
  ChoiceLayout* p = (ChoiceLayout*)layout;
  
  Parameter* cntlPar = p->control();

  // Note: LYTReader should not allow wrong contol type
  if(cntlPar->getType() == Parameter::StringChoice) {
    ChoiceFrame* frame =
      new ChoiceFrame((StringChoiceParameter*)cntlPar,_parent);
    
    list<Layout*>::const_iterator it = p->items().begin();
    while(it != p->items().end()) {
      const char* name = (*it)->name().c_str();
      GuiBuilder* b = new GuiBuilder(frame->contents(),_labelSize);      
      (*it++)->build(b);
      frame->addWidget(name,b->widget());
      
    }
    if (frame->layout()) frame->layout()->setMargin(_margin);
    _widget = frame;
  }
}

