/****************************************************************************
 *
 *  OnDemandScrollView.h                       XMM Science Analysis Subsystem
 *
 *  On-demand scroll-bars when widget exceeds a certain size
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: OnDemandScrollView.h,v 1.1 1999/07/20 08:09:20 jbrumfit Exp $
 *
 *****************************************************************************/

#ifndef ONDEMANDSCROLLVIEW_H
#define ONDEMANDSCROLLVIEW_H

#include <qwidget.h>
#include <qscrollview.h>
#include <qsize.h>


//================================================================
//  Class OnDemandScrollView
//
//  A subclass of QScrollView which has a default size, given by sizeHint(),
//  which is just large enough to accomodate the contents, without requiring 
//  scroll-bars. Subsequently resizing the widget to a smaller size will 
//  introduce scroll bars. The default size can be restricted to a specified
//  maximum height, width or both, in which case the default size may require 
//  scroll bars.
//
//  A typical use would be in a dialog whose contents are generated dynamically
//  and might occassionally become too large. In this case, the initial dialog
//  size should be determined by the contents. This can be achieved by calling
//  adjustSize() in the dialog's constructor.
//
//  The scroll-view has a single child, which is set by setContents().
//
//  Example:
//
//      OnDemandScrollView* scroller = new OnDemandScrollView(this);
//      scroller->setFrameStyle(QFrame::Box|QFrame::Raised);
//      scroller->setMaximumPreferredSize(200,200);
//
//      QWidget* contents = new QWidget(this);
//      QVBoxLayout* box = new QVBoxLayout(contents,10);
//      ... Add child widgets to contents
//
//      scroller->setContents(contents);
//
//================================================================

class OnDemandScrollView : public QScrollView
{
    Q_OBJECT

public:

    // Constructor
    OnDemandScrollView(QWidget *parent=0, const char *name=0);

    // Set a single widget as contents
    void setContents(QWidget* w);

    // Geometry management
    QSize sizeHint() const;
    QSize minimumSizeHint() const;
    void setMaximumPreferredWidth(int width);
    void setMaximumPreferredHeight(int height);
    void setMaximumPreferredSize(const QSize& size);
    void setMaximumPreferredSize(int width, int height);

private:

    // Member variables
    QWidget* _contents;
    int      _maxWidth;
    int      _maxHeight;

    // Constants
    static const int _minSize = 10;
};

#endif

