/*****************************************************************************
 *
 *  SasMainWindow.cc                            XMM Science Analysis Subsystem
 *
 *  Main window for the SAS GUI
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: SasMainWindow.cc,v 1.47 2000/11/23 16:31:57 jbakker Exp $
 *
 *****************************************************************************/

#include <assert.h>             //  headers

#include <iostream.h>           // C++ headers
#include <strstream.h>
#include <string>
#include <list>
#include <algorithm>
#include <set>
#include <Task.h>

#include <qpopupmenu.h>         // Qt headers
#include <qtooltip.h>
#include <qpixmap.h>
#include <qdir.h>
#include <qlayout.h>
#include <qfiledialog.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qsplitter.h>
#include <qwhatsthis.h>

#include "SasMainWindow.h"      // Application headers
#include "SasGuiGlobal.h"
#include "Logger.h"
#include "System.h"
#include "SasErrorHandler.h"
#include "TaskBrowser.h"
#include "HelpViewer.h"

#include <ParamInfoSet.h>
#include <errstr.h>
#include <XpiInfoReader.h>
#include <Error.h>
#include <LongMenu.h>
#include <AlertDialog.h>
#include <LYTReader.h>
#include <LayoutPrinter.h>
#include <SasFileLocator.h>
#include <DataSetBrowser.h>
#include <ParamDialog.h>

#include "icons/fileopen.xpm"    // Icons
#include "icons/filesave.xpm"
#include "icons/printicon.xpm"


//================================================================
//  Class SasMainWindow
//================================================================

//----------------------------------------
//  Constructors & destructors
//----------------------------------------

/*** Constructor ***/

SasMainWindow::SasMainWindow( const char *name )
    : SasTaskWindow(name)
{
    /*** Register the error handler ***/

    SasErrorHandler* handler = new SasErrorHandler();
    _oldHandler=errHandler.handler(handler);    


    /*** Initialise user preferences ***/

    initPreferences();


    /*** Set up printer ***/

    _printer = new QPrinter;
    _printer->setMinMax(1, 10);

    // TODO: move
    //    /*** Load application icon ***/
    //
    //    string logo = SasGuiGlobal::instance()->guiConfigDir() 
    //                  + "/xmmsaslogo.ppm";
    //    QPixmap pm;
    //    if (pm.load(logo.c_str()))
    //        setIcon(pm);


    /*** Create the task manager ***/

    _taskManager = new TaskManager(this);
    _taskManager->setPreferences(GuiPreferences::instance());
    connect(_taskManager,SIGNAL(enableKill(bool)),SLOT(enableKill(bool)));
    connect(_taskManager,SIGNAL(transientMessage(const char*)),
	    SLOT(transientMessage(const char*)));


    /*** Create the tool bar ***/

    _toolBar = new QToolBar(this, "toolbar");
    QToolButton* toolb;

    // Add browser button
    QPixmap browseIcon = QPixmap(fileopen);
    toolb = new QToolButton(browseIcon, "Browse dataset", 0,
			    this, SLOT(browseDataset()),
			    _toolBar, "browsetool" );
    QToolTip::add(toolb,"Browse datasets");
    QWhatsThis::add(toolb, "This button opens a <b>dataset browser</b>, which "
		    "allows you to inspect SAS datasets.");

    // Add print button
    QPixmap printIcon = QPixmap(printicon);
    toolb = new QToolButton(printIcon, "Print log", 0,
			    this, SLOT(print()),
			    _toolBar, "printtool" );
    QToolTip::add(toolb,"Print the log");
    QWhatsThis::add(toolb, "This button prints the current contents of the "
		    "<b>SAS log</b>, which is displayed in the log pane.");

    // Save icon, used in File menu
    QPixmap saveIcon = QPixmap(filesave);

    // Add "what's this" button
    toolb = QWhatsThis::whatsThisButton(_toolBar);
    QToolTip::add(toolb,"Obtain help on item");
    QWhatsThis::add(toolb, "This is the <b>Help Button</b> which provides help "
		    "on the various components of the window. Click on this button "
		    "and then click on an item to obtain help.");


    /*** Get debug mode ***/

    bool guiDebugMode = getenv("SAS_GUI_DEBUG") != 0;


    /*** Create the menus ***/
    
    _fileMenu = new QPopupMenu;
    CHECK_PTR(_fileMenu);
    _fileMenu->insertItem("Preferences...",  this, SLOT(editPreferences()));
    _fileMenu->insertSeparator();
    _fileMenu->insertItem(browseIcon, "Browse dataset...",  
			  this, SLOT(browseDataset()));
    _fileMenu->insertItem(printIcon, "Print log...",
			  this, SLOT(print()));
    _fileMenu->insertItem(saveIcon, "Save as script...", 
			  this, SLOT(saveAsScript()));
    _fileMenu->insertSeparator();
    _exitID = _fileMenu->insertItem("Exit",  this, SLOT(exit()));

    _taskMenu = new QPopupMenu;
    CHECK_PTR(_taskMenu);
    _taskMenu->insertItem("Revert to defaults", this, SLOT(resetParamInfo()));
    _killItem =_taskMenu->insertItem("Kill task...", this, SLOT(killTask()));
    _taskMenu->setItemEnabled(_killItem,false);

    QPopupMenu* customMenu = new QPopupMenu;
    CHECK_PTR(customMenu);

    QPopupMenu* testMenu = setupTestMenu();
    CHECK_PTR(testMenu);

    /*** Help on Packages ***/
    helpMenu()->insertItem("&Tasks", this, SLOT(helpTasks()), CTRL+Key_T,-1,0);
    

    /*** Create the menu bar and add the menus ***/

    QMenuBar* _menuBar = menuBar();
    if(guiDebugMode)
        _menuBar->insertItem("Test", testMenu,-1,0);
    _menuBar->insertItem("&Custom", customMenu,-1,0);
    _menuBar->insertItem("&Task", _taskMenu,-1,0);
    _menuBar->insertItem("&File", _fileMenu,-1,0);

    
    /*** Split main pane between TaskBrowser and LogPane ***/

    QSplitter* splitter = new QSplitter(this);
    splitter->setOrientation(QSplitter::Vertical);


    /*** Create the task browser ***/

    TaskBrowser* taskBrowser = new TaskBrowser(splitter);
    connect(taskBrowser, SIGNAL(run(const char*)), 
		SLOT(editParam(const char*)));
    QWhatsThis::add(taskBrowser, "This is the <b>Task browser</b>. Double click "
		    "on a task to edit the task parameters and run the task."
		    "Click on the column headings to sort the tasks in various "
		    "ways. To find a task quickly, you can type the first few "
		    "letters of the task name; the task browser must first be "
		    "selected by clicking anywhere in the pane.");
 

    /*** Create the log pane and enable logging ***/

    _logPane = new LogPane(splitter);
    CHECK_PTR(_logPane);
    _logPane->setLogFile("sas_log");
    Logger::registerLogHandler(_logPane);
    QWhatsThis::add(_logPane, "This is the <b>Log Pane</b> which records the execution "
		    "of each task as an equivalent command-line operation. <p>"
		    "<ul><li>Commands are prefixed by \"@@\"</li>"
		    "<li>Errors/warnings are prefixed by \"**\".</li>"
		    "<p>The log is automatically written to a log file in the current "
		    "directory.<p> The log may be saved as a command-line script with "
		    "the \"Save as script\" option on the File menu.</ul>");


    /*** Configure the window ***/

    setCentralWidget(splitter);
    statusBar();
    QWhatsThis::add(statusBar(), "This is the <b>Status Bar</b>. It is used to display "
		    "transient messages which do not need to be logged.");

	/* PATCH 1.28.4: default focus to TaskBrowser!!*/
	taskBrowser->setFocus();
}


/*** Destructor ***/

SasMainWindow::~SasMainWindow()
{
  // re-install the default handler
  errHandler.handler(_oldHandler);
}


//----------------------------------------
//  Initialisation
//----------------------------------------

/*** Initialisation which must be carried out after constructor ***/

void SasMainWindow::init()
{
    /*** Write a log message ***/

    Logger::write(string("XMM Science Analysis System - GUI version ") + 
		  Info::Task::current()->version());
		  //		  +  SasGuiGlobal::instance()->guiVersion());
    Logger::write(string("Started on ") + System::getDateTimeString());
    connect(GuiPreferences::instance(),SIGNAL(environmentChanged(const string&)),
	    this,SLOT(writeCommandLog(const string&)));
    GuiPreferences::instance()->setEnv(true);
}


//----------------------------------------
//  User preferences
//----------------------------------------

/*** Initialise user preferences [ PRIVATE ] ***/

void SasMainWindow::initPreferences()
{
    GuiPreferences::instance();
}


/*** Pop-up preferences dialog [ PRIVATE SLOT ] ***/

void SasMainWindow::editPreferences()
{
  ParamDialog dialog(GuiPreferences::instance()->parameters(), 0, 0,
		     true, "Ok", "sas/preferences");
  if (dialog.exec()) GuiPreferences::instance()->setEnv(false);
}

void SasMainWindow::writeCommandLog(const string& msg)
{
    Logger::writeCommand(msg);
}

/*** Reload all parameters (simply clears cache) [ PRIVATE SLOT ] ***/

void SasMainWindow::resetParamInfo()
{
    ParamInfoSet::instance()->resetParamInfo();
}


/*** Open parameter dialog and run task [ PRIVATE SLOT ] ***/

void SasMainWindow::editParam(const char* taskName)
{
    if(taskName)
        _taskManager->editParam(taskName);
}


/*** Kill the most recently started task ***/
    
void SasMainWindow::killTask()
{
    _taskManager->killLastTask();
}


/*** Enable 'Kill' on menu ***/

void SasMainWindow::enableKill(bool enable)
{
    _taskMenu->setItemEnabled(_killItem,enable);
}


//----------------------------------------
//  Application
//----------------------------------------

/*** Exit program [ PRIVATE SLOT ] ***/

void SasMainWindow::exit()
{
    if(_taskManager->tasksRunning())
    {
        if(QMessageBox::warning(this,name(),
	       "Cannot stop SAS GUI.\n There are tasks running") == 0)
        {
            return;
	}
    }

    Logger::write(string("\nSession ended ") + System::getDateTimeString());

    SasTaskWindow::quit();
}


/*** Display "Not yet implemented" message [ PRIVATE SLOT ] ***/

void SasMainWindow::notImplemented()
{
    errstr << "not yet implemented" << Sas::warning("notImplemented");
}    


//----------------------------------------
//  Dataset browser
//----------------------------------------

/*** Browse datasets [ PRIVATE SLOT ] ***/

void SasMainWindow::browseDataset()
{
    DataSetBrowser* dsv = new DataSetBrowser(this);
    dsv->setSelectable(DalListViewable::None);
    dsv->setCaption("DataSet Browser");
    dsv->exec();
    delete dsv;
}


//----------------------------------------
//  Printing
//----------------------------------------

/*** Print the log pane [ PRIVATE SLOT ] ***/

void SasMainWindow::print()
{
    _logPane->print(_printer, statusBar());
}


//----------------------------------------
//  Message pane
//----------------------------------------

void SasMainWindow::transientMessage(const char* text)
{
    statusBar()->message(text);
}


//----------------------------------------
//  Scripting
//----------------------------------------

/*** Save the log as a shell script ***/

void SasMainWindow::saveAsScript()
{
    _logPane->saveAsScript();
}

void SasMainWindow::helpTasks()
{
  string file = fileLocator.locate("doc/packages.html");
  HelpViewer().browse("file://" + file);
}


//----------------------------------------
//  Testing (Temporary)
//----------------------------------------

QPopupMenu* SasMainWindow::setupTestMenu()
{
    QPopupMenu* testMenu = new QPopupMenu;

    testMenu->insertItem( "Warning", this, SLOT(testWarning()));
    testMenu->insertItem( "Error", this, SLOT(testError()));
    testMenu->insertItem( "Fatal", this, SLOT(testFatal()));
    testMenu->insertItem( "Message 1", this, SLOT(testMessage1()));
    testMenu->insertItem( "Message 2", this, SLOT(testMessage2()));
    testMenu->insertSeparator();
    testMenu->insertItem( "test1", this, SLOT(test1()));

    return testMenu;
}


void SasMainWindow::testWarning()
{
    errstr << "This is to test warnings" << Sas::warning(string("warning"));
}

void SasMainWindow::testError()
{
    errstr << "This is to test errors" << Sas::error(string("error"));
}

void SasMainWindow::testFatal()
{
    errstr << "This is to test fatal errors" << Sas::fatal(string("fatal"));
}

void SasMainWindow::testMessage1()
{
    errstr << "This is a test message\nverbosity = Sparse" 
	   << Sas::message(Msg::App,Msg::Sparse);
}

void SasMainWindow::testMessage2()
{
    errstr << "This is to test message\nverbosity = Verbose" 
	   << Sas::message(Msg::App,Msg::Verbose);
}


/*** For temporary tests during development ***/

void SasMainWindow::test1()
{
    notImplemented();
}
