/*****************************************************************************
 *
 *  SasTask.h                                      XMM Science Analysis Subsystem
 *
 *  SAS SasTask control
 *
 *  (c) 1999 ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: SasTask.h,v 1.2 2000/10/24 09:36:59 ulammers Exp $
 *
 *****************************************************************************/

#ifndef SasTask_H
#define SasTask_H

#include <string>
#include <qobject.h>

#include "Process.h"
#include "ParameterInfo.h"
#include "GuiPreferences.h"
#include <MessageFormatter.h>

class ParameterInfo;


//=========================================================================
//  Class SasTask
//
//  Responsibilities:
//    - proxy for SAS task
//    - closure with parameters (which can be queued)
//    - running and killing SasTask
//    - asynchronous notification of messages
//    - asynchronous notification of completion
//
//  An instance of SasTask starts and controls an SAS task, with a given set of
//  parameters. The constructor creates a SasTask object, with a copy of the 
//  parameters, which may be queued for later execution.
//
//  Example:
//
//      task = new SasTask(paramInfo, this);
//      conect(task,SIGNAL(done(int,SasTask*)),SLOT(done(int,SasTask*)));
//      task->execute();
//
//      Demo::done() { delete task; }
//
//  The task notifies completion with a Qt signal 'done', which returns the 
//  exit code from the process(exitCode). If the SasTask is killed, the 'done'
//  signal is still generated, but with an exit code of -1.
//
//  The implementation currently forks a separate process and passes the 
//  parameters to it via the command line. An alternative implementation could
//  start a new thread and call the task as a function.
//
//==========================================================================

class SasTask : public QObject, public MessageHandler
{
    Q_OBJECT

public:
    // Types
    enum MessType {Warning, Error, Fatal, Info};
    
    // Constants
    static const char EOM = '\r';

    // Constructors and destructors
    SasTask(ParameterInfo* info, QObject* parent=0, GuiPreferences* pref=0);
    ~SasTask();

    // SasTask control
    void execute();
    void kill();
  
    // Information
    const char* name() { return _parameters->taskName().c_str(); }

signals:
    void done(int exitCode, SasTask* task);

private slots:
    void processDone(int code);
    void outputReceived(const char*);
    void errorReceived(const char*);

private:
    // Message Handling
    void infoMessage(const string& task, const string& text);
    void warningMessage(const string& task, const string& code, const string& text);
    void errorMessage(const string& task, const string& code, const string& text);
    void fatalMessage(const string& task, const string& code, const string& text);

    void parseStderr(const char* message);
    void setupTaskEnvironment();

    // Disable copy constructor and operator=
    SasTask(const SasTask&) 
        : QObject(), MessageHandler() {}
    SasTask& operator=(const SasTask&) {return *this;}

    // Member data
    Process*          _process;
    string            _messageBuffer;
    bool              _buffering;
    MessType          _messageType;
    string            _errcode;
    MessageFormatter* _formatter;
    ParameterInfo*    _parameters;
    GuiPreferences*   _preferences;
};

#endif

