/*****************************************************************************
 *
 *  TaskBrowser.cc                              XMM Science Analysis Subsystem
 *
 *  Browser for selecting tasks
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: TaskBrowser.cc,v 1.8 2000/11/23 16:31:57 jbakker Exp $
 *
 *****************************************************************************/

#include "TaskBrowser.h"       // Application headers
#include "SasGuiGlobal.h"
#include "LongMenu.h"
#include "QuotedString.h"
#include <errstr.h>

//#include <iostream.h>          // C++ headers
#include <fstream.h>
#include <string>
#include <list>
//#include <set>

#include <qframe.h>            // Qt headers
#include <qlistview.h>
#include <qapplication.h>
#include <qlayout.h>
#include <qdir.h>
#include <qstring.h>
#include <qheader.h>
#include <qwhatsthis.h>


//================================================================
//  Class TaskInfo
//================================================================
QString TaskInfo::group()
{
  int idx=_groups.find(' ');
  if (idx == -1) return _groups;

  return _groups.left(idx);
}


void TaskInfo::read(const QString& file)
{
  ifstream is(file.latin1());
  errstr << " info-file: " << file
	 << Sas::message(Msg::Meta, Msg::Noisy);

  int linenumber=0;
  while (is.good()) {
    const int ln=2048; // hard limit
    char line[ln+1];line[ln]=0;
    if (!is.getline(line,ln)) break;
    linenumber++;
    
    // skip comments, empty lines, etc...
    QString s(line);
    s=s.stripWhiteSpace();
    if (s.isEmpty()) continue;
    if (s[0] == '#') continue;
    
    // check whether we have any KEYWORD=at all?
    int i=s.find('=');
    if (i==-1) {
      errstr << "  ignore line " << linenumber 
	     << ":: wrong syntax"
	     << Sas::message(Msg::Meta, Msg::Noisy);
      continue;
    }

    QString key=s.left(i).stripWhiteSpace();
    QString val=s.right(s.length()-i-1).stripWhiteSpace();
    
    if      (key == "DESCRIPTION") _description=val;
    else if (key == "VISIBLE")     _visible=(val=="yes");
    else if (key == "GROUPS")      _groups=val;
    else    {
      errstr << "  ignore line "  << linenumber 
	     << ":: invalid key:[" << key << "]"
	     << Sas::message(Msg::Meta, Msg::Noisy);
    }
  }
  is.close();
}


//================================================================
//  Class TaskBrowser
//================================================================

/*** Constructor ***/

TaskBrowser::TaskBrowser(QWidget *parent, const char *name)
  :  QListView(parent,name)
{
    _history = 0;

    QWhatsThis::add(header(), "Click on column headings to sort tasks according "
		    "to that category. Click again to sort in reverse order. "
		    "Click on the history column to show recently used tasks.");


    /*** Set up columns for list view ***/

    addColumn("task");
    addColumn("type");
    addColumn("history");
    addColumn("description");

    setFrameStyle(QFrame::WinPanel + QFrame::Sunken);
    setSorting(0);  // Use -1 to disable

    connect(this,SIGNAL(doubleClicked(QListViewItem*)),
	    SLOT(itemDoubleClicked(QListViewItem*)));
    connect(this,SIGNAL(returnPressed(QListViewItem*)),
	    SLOT(itemDoubleClicked(QListViewItem*)));

    setup();            // Scan config files and add tasks which are found
}

/* creates all items ins the browser */
void TaskBrowser::setup()
{
  /* read all necessary information */
  readTaskPar();
  /*
    TODO:delete obsolete method!
    readOldTaskInfo();
  */
  readTaskInfo();

  /* show only those task that are visible */
  map< QString, TaskInfo* >::const_iterator it=_info.begin();
  for (;it!=_info.end();it++) {
    TaskInfo* info=(*it).second;
    if (!info) continue;
    if (!info->visible()) continue;
    
    new QListViewItem(this,
		      (*it).first,
		      info->group(),
		      "",
		      info->description()
		      );
  }
}

/*
 The config directories in the SAS_PATH are searched for parameter
 files. If present, the tasks are added to the info map.
*/
void TaskBrowser::readTaskPar()
{
  /*
    At this instance we fill the map with all possible tasks,
    as determined by available task.par files
   */
  list<string> sasDirs = SasGuiGlobal::instance()->sasPath();
  
  for (list<string>::const_iterator i = sasDirs.begin();
       i != sasDirs.end(); ++i)
    {
      const string& topDir = (*i);
      
      /*** Try to access the directory ***/
      
      string configDir = topDir + "/config";
      QDir d(configDir.c_str());
      if (!d.exists())
	continue;
      

        /*** Add all tasks from this config directory ***/
        d.setNameFilter("*.par");
        d.setSorting(QDir::Name);

        const QFileInfoList *list = d.entryInfoList();
        QFileInfoListIterator it(*list);
        QFileInfo *fi;
    
        while ( (fi=it.current()) )
        {
            QString name = fi->baseName();

	    TaskInfo* info= _info[name];
	    if (! info) _info[name]=new TaskInfo();
            ++it;
	}
    }
}


// Notify when a task is double-clicked
void TaskBrowser::itemDoubleClicked(QListViewItem* item)
{
    item->setText(2,QString::number(++_history));
    emit run(item->text(0));
}


// Read the "task_info" file to build an initial list of task data.
void TaskBrowser::readOldTaskInfo()
{
    string fileName = SasGuiGlobal::instance()->guiConfigDir() 
                      + "/tasks.info.obs";

    ifstream is(fileName.c_str());
    while(is)
    {
        string nameOld;
	string typeOld;
	QuotedString descriptionOld;

	is >> nameOld >> typeOld >> descriptionOld;

	QString name(nameOld.c_str());
	QString group(typeOld.c_str());
	QString description("<old:");
	description+=descriptionOld.c_str();
	description+=">";

	TaskInfo* info= _info[name];
	if (info) *info=TaskInfo(description,group);
    }
    is.close();
}

// Read all available '.info' files
void TaskBrowser::readTaskInfo()
{
  list<string> sasDirs = SasGuiGlobal::instance()->sasPath();
  
  /* PATCH 1.28.4
   *   reverse read, such that first dir in SAS_PATH overrides older
   * TODO:
   *   normal iterator,and add switch to TaskInfo that tells to skip
   */
  for (list<string>::reverse_iterator i = sasDirs.rbegin();
       i != sasDirs.rend(); ++i)
  /*
  for (list<string>::const_iterator i=sasDirs.begin();i!=sasDirs.end();++i)
  */
    {
      QString configDir = (*i).c_str();
      configDir +="/config";
      QDir d(configDir);
      if (!d.exists()) 	continue;
      

      /*** Add all tasks from this config directory ***/
      d.setNameFilter("*.info");
      d.setSorting(QDir::Name);
      
      const QFileInfoList *list = d.entryInfoList();
      QFileInfoListIterator it(*list);
      QFileInfo *fi;
    
      while ( (fi=it.current()) )
        {
	  /* read info file*/
	  QString name = fi->baseName();
	  TaskInfo* info= _info[name];
	  if (info) _info[name]->read( fi->absFilePath() );
	  ++it;
	}
    }

}

